package jaugment2.worldclock;


  //JDK
import java.rmi.server.UnicastRemoteObject;
import java.rmi.MarshalledObject;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.Date;
import java.util.TimeZone;
import java.util.Set;
import java.util.Iterator;
import java.text.DateFormat;
import java.net.URL;
  //JINI
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
import net.jini.core.lease.Lease;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.lease.LeaseRenewalManager;
  //Project
import jaugment2.speech.SemiNaturalInputProcessing;
import jaugment2.speech.SemiNaturalInputProcessing.SentenceParsedEvent;
import jaugment2.speech.SemiNaturalInputProcessing.Word;
import jaugment2.helpers.Logger;





public class SemiNatRepresentation
{
  private LeaseRenewalManager lrm = new LeaseRenewalManager();
  private static final boolean debug = true;
  private WorldClockService proxy;

  public SemiNatRepresentation(WorldClockService p) throws RemoteException
  {
    this.proxy = p;
  }




    /**
     * register at a SemiNaturalInputProcessing
     */
  public void addProcessor(SemiNaturalInputProcessing p) throws RemoteException
  {
    if(debug)Logger.debug.println("DEBUG: [worldclock.SemiNatRepresentation] addProcessor(...)");
    try
    {
      this.getTimeListener = new WorldTimeListener(proxy);
      String s[] = null;
      String fullForm = System.getProperty("jaugment2.worldclock.SemiNatRepresentation.gettime.fullForm", "what time is in");
      String shortForm = System.getProperty("jaugment2.worldclock.SemiNatRepresentation.gettime.shortForm", "time in");
      Set cities = proxy.cities();
      s = new String[cities.size()*2];
      int counter = 0;
      for ( Iterator i=cities.iterator(); i.hasNext(); )
      {
        String cityName = (String)i.next();
        s[counter++] = fullForm+" "+cityName;
        s[counter++] = shortForm+" "+cityName;
      } // end of for ()
      
      for(int i=0; i<s.length; i++)
      {
        if(debug)Logger.debug.println("DEBUG: [worldclock.SemiNatRepresentation] add sentence:"+s[i]);
        Lease l = p.addSentenceParsedListener(this.getTimeListener, null,s[i]);
        lrm.renewFor(l, Lease.FOREVER, null);
      }

    }
    catch(LeaseDeniedException e)
    {
      Logger.err.println("ERROR: [SemiNatRepresentation] addProcessor(...) - adding us as a listener for our sentences was denied by the service");
    }
  }



  private WorldTimeListener getTimeListener = null;

  private class WorldTimeListener implements RemoteEventListener , java.io.Serializable
  {

    private static final boolean debug = true;
    private WorldClockService service;
    public WorldTimeListener(WorldClockService service) throws RemoteException
    {
      this.service = service;
    }

      /**
       * react to an event from a SemiNaturalInputProcessing
       */
    public void notify( RemoteEvent p0)
      throws UnknownEventException, RemoteException
    {
      if(debug)Logger.debug.println("DEBUG: [clock.SemiNatRepresentation.WorldTimeListener] the following sentence has been given to us: \""+p0.toString()+"\"");
      try
      {
        if(!(p0 instanceof SentenceParsedEvent))
          throw new UnknownEventException("expecting SentenceParsedEvent");

        SentenceParsedEvent evt = (SentenceParsedEvent)p0;

        Word words[] = evt.getWords();

        if(debug)Logger.debug.println("DEBUG: [clock.SemiNatRepresentation.getTimeListener] the following sentence has been given to us: \""+p0.toString()+"\"");

          //TODO
        DateFormat timeFormat = DateFormat.getTimeInstance();
        String timeZone = (String)service.getTimeZone(words[words.length-1].toString());
        String answer = "I don't understand.";
        if ( timeZone != null )
        {
          timeFormat.setTimeZone( TimeZone.getTimeZone( timeZone));
          answer = "It is "+timeFormat.format(new Date())+".";
        } // end of else
        if(debug)
          Logger.debug.println("DEBUG: [clock.SemiNatRepresentation.getTimeListener] answer: \""+answer+"\"");
        evt.answer(answer);
      }
      catch(Exception e)
      {
        e.printStackTrace(Logger.err);
      }
    }
  };




}
