package jaugment2.worldclock;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;
import java.util.Map;
import java.util.Set;
import java.net.URL;
import java.net.InetAddress;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;

import net.jini.core.entry.Entry;
import net.jini.core.discovery.LookupLocator;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
  //project
import jaugment2.speech.SemiNaturalInputProcessing;
import jaugment2.jini.simpleService.simpleService;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;
import jaugment2.jini.LookupEventListener;
import jaugment2.jini.LookupEventEvent;




//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


public class WorldClockService
  extends simpleService
  implements Remote  , LookupEventListener
{

  private static final boolean debugMessages = false;
  
    //--------------------- initiaslisation&administration ------------------------------------------------------//

  private WorldClockServiceProxy myproxy;
  Hashtable cityZones = null;//new Hashtable();
  

    /**
     * Load our state (e.g. attrubutes, service-id) from persistent storage
     */
  private void loadMe()
  {
    String filename = System.getProperty("jaugment2.worldclock.WorldClockService.persistentStateFilename");
    if(filename==null)
    {
      filename = "WorldClockServiceState.dat";
      System.setProperty("jaugment2.worldclock.WorldClockService.persistentStateFilename", filename);
    };

    persistentState.setProperty("persistentStateFilename", filename);

    load();


    if(persistentState.get("attr")==null)
    {
      Entry[] attrSets = new Entry[]
      {
        new net.jini.lookup.entry.Name("WorldClockService"),
        new net.jini.lookup.entry.Comment("a service providing World time and timezone related data"),
        new net.jini.lookup.entry.ServiceInfo("WorldClockService", "Eugene Kireyev<eugene_kireyev@hotmail.com>", "Marcus Wolschon <Suran@gmx.net>, Eugene Kireyev<eugene_kireyev@hotmail.com>", "0.0.1", "standart", "00000"),

        WorldClockServiceRepresentation.getUIDescriptor()
      };
      persistentState.put("attr", attrSets);
    };

    save();
  };


  public WorldClockService() throws RemoteException
  {
    this(null);
  };

  public WorldClockService(LookupLocator locallocator) throws RemoteException
  {
    if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] creating myself...");
    init();
    try
    {
      landLord = new SimpleLandlord(simpleLeaselistener);
      this.seminat = new SemiNatRepresentation(this);

      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");

      myproxy = new WorldClockServiceProxy(this);

      loadMe();

      Entry[] attrSets = (Entry[])persistentState.get("attr");

      this.init(myproxy, attrSets, this, locallocator);
    }
    catch(Exception e)
    {
      e.printStackTrace(Logger.err);
    };
  };



  
  private void init()
  {
   if(debugMessages) Logger.debug.println("DEBUG: [WorldClockService] init()");
LinkedList cities = (LinkedList)System.getProperties().get("jaugment2.worldclock.WorldClockService.cities");
    try 
    {
      cityZones = new Hashtable();
      for(int i=0; i<cities.size(); i++)
      {
        LinkedList city = (LinkedList)cities.get(i);
        String cityName = (String)city.get(0);
        String cityTimeZone = (String)city.get(1);
        if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] city :"+cityName+", zone="+cityTimeZone);
        cityZones.put( cityName, cityTimeZone);
      }
      
    }
    catch ( NullPointerException e)
    {
      e.printStackTrace( Logger.err);
      
        //nothing special, maybe some list isn't valide
    } // end of try-catch
    

  }
/*  
  public Map getCities()
  {
    Logger.debug.println("DEBUG: [WorldClockService] getCities()");
    
    return cityZones;
  }
*/
  public Set cities()
  {
   if(debugMessages) Logger.debug.println("DEBUG: [WorldClockService] getCities()");
    
    return cityZones.keySet();
  }

  public String getTimeZone( String cityName)
  {
    return (String)cityZones.get(cityName);
  }
  
    /** restart this service once it has been stopped
     * @see stop
     */
  public void restart() throws Exception
  {
   super.restart();
  }

    /** stop this service. If exit is false all references to this
     * service are still valid but it has deregistered everywhere.
     * (this includes the RMI-activation-daemon)
     * @see restart
     * @see destroy
     * @param exit stop running
     */
  public void stop(boolean exit) throws Exception
  {
   super.stop(exit);
  }

    /** stop this service, destroy any persistent storage and exit
     */
  public void destroy() throws RemoteException
  {
   super.destroy();
  }

    //-------------------------- registrations -----------------------------------


    //-------------------- leases

    /**
     * Class for events thrown here
     */
  public static class DirectoryServiceEvent extends RemoteEvent//TODO extends MailEvent
  {
//  public static final int EVID_NEWTEXTEDITORPROXY = 1;
//TODO

    public DirectoryServiceEvent( Object proxy, long eventID, long seqNum,
                                  MarshalledObject handback)
    {
      super(proxy, eventID, seqNum, handback);
    }

  }

  private LinkedList listenerRegistrations[] = new LinkedList[]{new LinkedList(),new LinkedList()};

    /**
     * Helper-class for our Landlord.
     */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
    public void leaseCanceled(int type, Object reg, int newcount)
    {
//    EventRegistration reg2 = (EventRegistration)reg;
        //TODO
    };
  };

  private SimpleLandlord landLord;

  private void fireEvent(int type)
  {
    if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
    for(int i=0; i<listenerRegistrations[type].size(); i++)
    {
/*      try
        {                     //TODO: do this asyncronously
        EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
        Logger.debug.println("DEBUG: [WorldClockService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
        if( (entry.subscribedFor & type) != 0)
        entry.listener.notify(
        new DirectoryServiceEvent(getProxy(),
        type,
        entry.seq++,
        entry.handback
        )
        );
        }
        catch(RemoteException ex)
        {
        Logger.err.println("ERROR: [WorldClockService:fireEvent] could not deliver event");
        ex.printStackTrace(Logger.err);
        }
        catch(UnknownEventException ex)
        {
        Logger.err.println("ERROR: [WorldClockService:fireEvent] could not deliver event - the receiver does not know it");
        ex.printStackTrace(Logger.err);
        }*/
    }
  };

    /** Subscribe for events
     */
  public Lease addListener( RemoteEventListener listener,
                            MarshalledObject handback, int registerfor)
    throws RemoteException, LeaseDeniedException, IllegalStateException
  {
    if(listener==null)
      return(null);

    try
    {
      EventRegistration reg = new EventRegistration(listener, handback);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

      if(l != null)
      {
          // tokenize the expression
        if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] addListener expr=\""+registerfor+"\"");

        for(int i=1; i<(2*registerfor) && (i<listenerRegistrations.length); i=i*2)
          if((i & registerfor) == i)
          {
            listenerRegistrations[i].add(reg);
          };
      };

      return(l);
    }
    catch(Exception ex)
    {
      ex.printStackTrace(Logger.err);
    }
    return(null);
  };



    //-------------------------- functionality -----------------------------------

  private SemiNatRepresentation seminat = null;//new SeminatRepresentation(this);

  public void serviceFoundEvent(LookupEventEvent evt)
  {
    try
    {
      if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] serviceFoundEvent");
      if(evt.item != null)
      {
        if(evt.item.service != null)
        {
      if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] evt.item.service="+evt.item.service);
          if(evt.item.service instanceof SemiNaturalInputProcessing)
          {
            if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] found a SemiNaturalInputProcessing");
            if(this.seminat == null)
            {
              this.seminat = new SemiNatRepresentation(this);
            }
            seminat.addProcessor((SemiNaturalInputProcessing)evt.item.service);
          }
        }
      }
    }
    catch(Exception ex)
    {
      ex.printStackTrace(Logger.err);
    }
  }

  public void serviceGoneEvent(LookupEventEvent evt)
  {
    if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] serviceGoneEvent");
  }


  public void serviceChangedEvent(LookupEventEvent evt)
  {
    if(debugMessages)Logger.debug.println("DEBUG: [WorldClockService] serviceChangedEvent");
  }

}
